class Location(object):
    def __init__(self, x, y):
        self.x = x
        self.y = y

    def __add__(self, direction):
        return Location(self.x + direction.x, self.y + direction.y)

    def __sub__(self, direction):
        return Location(self.x - direction.x, self.y - direction.y)

    def __repr__(self):
        return "Location({}, {})".format(self.x, self.y)

    def __eq__(self, other):
        return self.x == other.x and self.y == other.y

    def __ne__(self, other):
        return not self == other

    def __hash__(self):
        return hash((self.x, self.y))



from simulation.location import Location


class AvatarState(object):
    def __init__(self, location, health, score, backpack):
        self.location = Location(**location)
        self.health = health
        self.score = score
        self.backpack = backpack



from abc import ABCMeta, abstractmethod


class Action:
    __metaclass__ = ABCMeta

    @abstractmethod
    def serialise(self):
        pass


class WaitAction(Action):
    def serialise(self):
        return {"action_type": "wait"}


class PickupAction(Action):
    def serialise(self):
        return {"action_type": "pickup"}


class MoveAction(Action):
    def __init__(self, direction):
        self.direction = direction

    def serialise(self):
        return {
            "action_type": "move",
            "options": {"direction": self.direction.serialise()},
        }


class AttackAction(Action):
    def __init__(self, direction):
        self.direction = direction

    def serialise(self):
        return {
            "action_type": "attack",
            "options": {"direction": self.direction.serialise()},
        }



# -*- coding: utf-8 -*-
from setuptools import find_packages, setup
import importlib.util


def get_version():
    try:
        version_spec = importlib.util.spec_from_file_location(
            "aimmo_version", "../aimmo/__init__.py"
        )
        aimmo_version_module = importlib.util.module_from_spec(version_spec)
        version_spec.loader.exec_module(aimmo_version_module)
        return aimmo_version_module.__version__
    except Exception:
        return "0.0.0"


setup(
    name="aimmo-avatar-api",
    packages=find_packages(include=["simulation"]),
    version=get_version(),
    include_package_data=True,
    tests_require=["httmock", "mock"],
    test_suite="tests",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)



from .avatar_state import AvatarState
from .location import Location


class Cell(object):

    """
    Any position on the world grid.
    """

    def __init__(self, location, avatar=None, **kwargs):
        self.location = Location(**location)
        self.avatar = None
        if avatar:
            self.avatar = AvatarState(
                location=avatar["location"],
                score=avatar["score"],
                health=avatar["health"],
                backpack=avatar["backpack"],
            )
        for (key, value) in kwargs.items():
            setattr(self, key, value)

    def has_artefact(self):
        return self.interactable is not None and self.interactable["type"] == "artefact"

    def __repr__(self):
        return "Cell({} h={} a={} i={})".format(
            self.location, self.habitable, self.avatar, self.interactable
        )

    def __eq__(self, other):
        return self.location == other.location

    def __ne__(self, other):
        return not self == other


class WorldMap(object):

    """
    The non-player world state.
    """

    def __init__(self, cells):
        self.cells = {}
        for cell_data in cells:
            cell = Cell(**cell_data)
            self.cells[cell.location] = cell

    def all_cells(self):
        return self.cells.values()

    def interactable_cells(self):
        return [cell for cell in self.all_cells() if cell.interactable]

    def pickup_cells(self):
        pickup_types = ("damage_boost", "invulnerability", "health", "artefact")
        return [
            cell
            for cell in self.interactable_cells()
            if cell.interactable["type"] in pickup_types
        ]

    def score_cells(self):
        return [
            cell
            for cell in self.interactable_cells()
            if "score" == cell.interactable["type"]
        ]

    def partially_fogged_cells(self):
        return [cell for cell in self.all_cells() if cell.partially_fogged]

    def is_visible(self, location):
        return location in self.cells

    def get_cell(self, location):
        cell = self.cells[location]
        assert (
            cell.location == location
        ), "location lookup mismatch: arg={}, found={}".format(location, cell.location)
        return cell

    def can_move_to(self, target_location):
        try:
            cell = self.get_cell(target_location)
        except KeyError:
            return False
        return getattr(cell, "habitable", False) and not getattr(cell, "avatar", False)

    def __repr__(self):
        return repr(self.cells)
